/*
* Google Search on ESP32 with JSON API
* 
* uPesy Electronics
* www.upesy.com
*/


#include <Arduino.h>
#include <WiFi.h>
#include <HTTPClient.h>
#define ARDUINOJSON_DECODE_UNICODE 1
#include <ArduinoJson.h>

#define USE_PSRAM 0 //1 For using PSRAM

//WiFi
const char *ssid = "SSID";
const char *password = "WiFi PASSWORD";

//URL Parameters
String cx = "YOUR_SEARCH_ENGINE_ID";
String apiKey = "YOUR_API_KEY";
String query = "";
String url = "https://customsearch.googleapis.com/customsearch/v1?key="+apiKey+"&cx="+cx+"&q=";

#if USE_PSRAM
struct SpiRamAllocator {
  void* allocate(size_t size) {
    return ps_malloc(size);
    
  }
  void deallocate(void* pointer) {
    free(pointer);
  }
};

using SpiRamJsonDocument = BasicJsonDocument<SpiRamAllocator>;

#endif


String urlencode(String str){
    String encodedString="";
    char c;
    char code0;
    char code1;
    for (int i =0; i < str.length(); i++){
      c=str.charAt(i);
      if (c == ' '){
        encodedString+= '+';
      } else if (isalnum(c)){
        encodedString+=c;
      } else{
        code1=(c & 0xf)+'0';
        if ((c & 0xf) >9){
            code1=(c & 0xf) - 10 + 'A';
        }
        c=(c>>4)&0xf;
        code0=c+'0';
        if (c > 9){
            code0=c - 10 + 'A';
        }
        encodedString+='%';
        encodedString+=code0;
        encodedString+=code1;
      }
      yield();
    }
    return encodedString;
}


void setup(){

  delay(500);
  Serial.begin(115200);

  #if USE_PSRAM
  psramInit();
  Serial.println((String)"Memory available in PSRAM : " +ESP.getFreePsram()+" B");
  #endif

  //Connect to WiFi
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED){
    delay(100);
    Serial.print(".");
  }

  Serial.print("\nWiFi connected with IP : ");
  Serial.println(WiFi.localIP());
  Serial.println("\n\nType your search request"); 
}

void loop(){
#if USE_PSRAM
  SpiRamJsonDocument doc(150000);
#else
  DynamicJsonDocument doc(100000);
#endif

  while(Serial.available()) {
    query = Serial.readString();// read the incoming data as string
    Serial.println("Searching : " + query);
    HTTPClient http;
    http.useHTTP10(true);
    http.begin(url+urlencode(query));
    http.GET();

    
    DeserializationError error = deserializeJson(doc, http.getStream());
    if(error){
      Serial.print("deserializeJson() failed: ");
      Serial.println(error.c_str());
    }

    http.end();

    for (int i=0; i!=10;i++){
      Serial.print("\n[+] ");
      Serial.println(doc["items"][i]["title"].as<String>());
      Serial.println("------------------------------");
      Serial.println(doc["items"][i]["snippet"].as<String>());
      Serial.println("------------------------------");
      Serial.println((String) "URL : " + doc["items"][i]["link"].as<String>());
    }
    Serial.println((String)"\nJson Size: " + doc.memoryUsage() + " B");
    doc.clear();
    Serial.println("\n\nType your search request");
  }

}
